@@ -1,43 +1,50 @@
+from __future__ import annotations
+
 import logging
 from dataclasses import dataclass, field
-from datetime import timedelta
-from typing import Any, Callable
+from datetime import timedelta  # noqa: TC003
+from typing import TYPE_CHECKING, Any, Callable
 
-import pandas as pd
 import polars as pl
 
 from aligned import AwsS3Config
-from aligned.compiler.feature_factory import FeatureFactory, Transformation, TransformationFactory
-from aligned.schemas.transformation import LiteralValue, TextVectoriserModel
+from aligned.lazy_imports import pandas as pd
+from aligned.compiler.feature_factory import (
+    FeatureFactory,
+    Transformation,
+    TransformationFactory,
+)
+from aligned.schemas.feature import FeatureReference, FeatureType
+from aligned.schemas.transformation import (
+    FillNaValuesColumns,
+    LiteralValue,
+    EmbeddingModel,
+    BinaryOperators,
+    Expression,
+)
+
+if TYPE_CHECKING:
+    from aligned.feature_store import ContractStore
 
 logger = logging.getLogger(__name__)
 
 
 @dataclass
-class EqualsFactory(TransformationFactory):
-
-    left: FeatureFactory
-    right: FeatureFactory | Any
+class IsNullFactory(TransformationFactory):
+    value: FeatureFactory
 
     @property
     def using_features(self) -> list[FeatureFactory]:
-        if isinstance(self.right, FeatureFactory):
-            return [self.left, self.right]
-        else:
-            return [self.left]
+        return [self.value]
 
     def compile(self) -> Transformation:
-        from aligned.schemas.transformation import Equals
+        from aligned.schemas.transformation import IsNull
 
-        if isinstance(self.right, FeatureFactory):
-            raise NotImplementedError()
-        else:
-            return Equals(self.left.name, LiteralValue.from_value(self.right))
+        return IsNull(Expression.from_value(self.value))
 
 
 @dataclass
 class NotNullFactory(TransformationFactory):
-
     value: FeatureFactory
 
     @property
@@ -47,30 +54,11 @@ class NotNullFactory(TransformationFactory):
     def compile(self) -> Transformation:
         from aligned.schemas.transformation import NotNull
 
-        return NotNull(self.value.name)
-
-
-@dataclass
-class RatioFactory(TransformationFactory):
-
-    numerator: FeatureFactory
-    denumerator: FeatureFactory | LiteralValue
-
-    @property
-    def using_features(self) -> list[FeatureFactory]:
-        return [self.numerator, self.denumerator]
-
-    def compile(self) -> Transformation:
-        from aligned.schemas.transformation import DivideDenumeratorValue, Ratio
-
-        if isinstance(self.denumerator, LiteralValue):
-            return DivideDenumeratorValue(self.numerator.name, self.denumerator)
-        return Ratio(self.numerator.name, self.denumerator.name)
+        return NotNull(Expression.from_value(self.value))
 
 
 @dataclass
 class OrdinalFactory(TransformationFactory):
-
     orders: list[str]
     feature: FeatureFactory
 
@@ -85,8 +73,37 @@ class OrdinalFactory(TransformationFactory):
 
 
 @dataclass
-class ArrayContainsFactory(TransformationFactory):
+class ArrayAtIndexFactory(TransformationFactory):
+    feature: FeatureFactory
+    index: int
 
+    @property
+    def using_features(self) -> list[FeatureFactory]:
+        return [self.feature]
+
+    def compile(self) -> Transformation:
+        from aligned.schemas.transformation import ArrayAtIndex
+
+        return ArrayAtIndex(self.feature.name, self.index)
+
+
+@dataclass
+class ArrayContainsAnyFactory(TransformationFactory):
+    values: LiteralValue
+    in_feature: FeatureFactory
+
+    @property
+    def using_features(self) -> list[FeatureFactory]:
+        return [self.in_feature]
+
+    def compile(self) -> Transformation:
+        from aligned.schemas.transformation import ArrayContainsAny
+
+        return ArrayContainsAny(self.in_feature.name, self.values)
+
+
+@dataclass
+class ArrayContainsFactory(TransformationFactory):
     value: LiteralValue
     in_feature: FeatureFactory
 
@@ -102,7 +119,6 @@ class ArrayContainsFactory(TransformationFactory):
 
 @dataclass
 class ContainsFactory(TransformationFactory):
-
     text: str
     in_feature: FeatureFactory
 
@@ -117,131 +133,106 @@ class ContainsFactory(TransformationFactory):
 
 
 @dataclass
-class NotEqualsFactory(TransformationFactory):
-
-    value: Any
-    in_feature: FeatureFactory
-
-    @property
-    def using_features(self) -> list[FeatureFactory]:
-        return [self.in_feature]
-
-    def compile(self) -> Transformation:
-        from aligned.schemas.transformation import NotEquals as NotEqualsTransformation
-
-        return NotEqualsTransformation(self.in_feature.name, LiteralValue.from_value(self.value))
-
-
-@dataclass
-class GreaterThenFactory(TransformationFactory):
-
-    left_feature: FeatureFactory
+class BinaryFactory(TransformationFactory):
+    left: Any
     right: Any
+    operation: BinaryOperators
 
     @property
     def using_features(self) -> list[FeatureFactory]:
+        features = []
+        if isinstance(self.left, FeatureFactory):
+            features.append(self.left)
+            features.extend(
+                feat
+                for feat in self.left.feature_dependencies()
+                if feat._name is not None
+            )
         if isinstance(self.right, FeatureFactory):
-            return [self.left_feature, self.right]
-        else:
-            return [self.left_feature]
+            features.append(self.right)
+            features.extend(
+                feat
+                for feat in self.right.feature_dependencies()
+                if feat._name is not None
+            )
+        return features
 
     def compile(self) -> Transformation:
-        from aligned.schemas.transformation import GreaterThen, GreaterThenValue
+        from aligned.schemas.transformation import BinaryTransformation, Expression
+
+        if isinstance(self.left, FeatureFactory):
+            left = self.left.to_expression()
+        else:
+            left = Expression(literal=LiteralValue.from_value(self.left))
 
         if isinstance(self.right, FeatureFactory):
-            return GreaterThen(self.left_feature.name, self.right.name)
+            right = self.right.to_expression()
         else:
-            return GreaterThenValue(self.left_feature.name, self.right)
+            right = Expression(literal=LiteralValue.from_value(self.right))
+
+        return BinaryTransformation(left, right, operator=self.operation)
 
 
 @dataclass
-class GreaterThenOrEqualFactory(TransformationFactory):
-
-    value: Any
-    in_feature: FeatureFactory
+class Split(TransformationFactory):
+    pattern: str
+    from_feature: FeatureFactory
 
     @property
     def using_features(self) -> list[FeatureFactory]:
-        return [self.in_feature]
+        return [self.from_feature]
 
     def compile(self) -> Transformation:
-        from aligned.schemas.transformation import GreaterThenOrEqual as GTETransformation
+        from aligned.schemas.transformation import Split as SplitTransformation
 
-        return GTETransformation(self.in_feature.name, self.value)
+        return SplitTransformation(
+            Expression.from_value(self.from_feature), self.pattern
+        )
 
 
 @dataclass
-class LowerThenFactory(TransformationFactory):
-
-    value: float
-    in_feature: FeatureFactory
+class OllamaEmbedding(TransformationFactory):
+    model: str
+    from_feature: FeatureFactory
+    host_env: str | None
 
     @property
     def using_features(self) -> list[FeatureFactory]:
-        return [self.in_feature]
+        return [self.from_feature]
 
     def compile(self) -> Transformation:
-        from aligned.schemas.transformation import LowerThen as LTTransformation
+        from aligned.schemas.transformation import (
+            OllamaEmbedding as OllamaEmbeddingTransformation,
+        )
 
-        return LTTransformation(self.in_feature.name, self.value)
+        return OllamaEmbeddingTransformation(
+            self.from_feature.name, self.model, self.host_env
+        )
 
 
 @dataclass
-class LowerThenOrEqualFactory(TransformationFactory):
-
-    value: float
-    in_feature: FeatureFactory
+class OllamaGenerate(TransformationFactory):
+    model: str
+    system: str | None
+    prompt_feature: FeatureFactory
+    host_env: str | None
 
     @property
     def using_features(self) -> list[FeatureFactory]:
-        return [self.in_feature]
+        return [self.prompt_feature]
 
     def compile(self) -> Transformation:
-        from aligned.schemas.transformation import LowerThenOrEqual as LTETransformation
+        from aligned.schemas.transformation import (
+            OllamaGenerate as OllamaGenerateTransformation,
+        )
 
-        return LTETransformation(self.in_feature.name, self.value)
-
-
-# @dataclass
-# class Split(TransformationFactory):
-
-#     pattern: str
-#     from_feature: FeatureFactory
-#     max_splits: int | None
-
-#     @property
-#     def method(self) -> Callable[[pd.DataFrame], pd.Series]:
-#         async def met(df: pd.DataFrame) -> pd.Series:
-#             return df[self.from_feature.name].str.split(pat=self.pattern, n=self.max_splits)
-
-#         return met
-
-#     def index(self, index: int) -> 'ArrayIndex':
-#         return ArrayIndex(index, self)
-
-
-# class ArrayIndex(DillTransformationFactory):
-
-#     index: int
-#     from_feature: FeatureFactory
-
-#     def __init__(self, index: int, feature: FeatureFactory) -> None:
-#         self.using_features = [feature]
-#         self.feature = Bool()
-#         self.index = index
-#         self.from_feature = feature
-
-#     @property
-#     def method(self) -> Callable[[pd.DataFrame], pd.Series]:
-#         async def met(df: pd.DataFrame) -> pd.Series:
-#             return df[self.from_feature.name].str[self.index]
-
-#         return met
+        return OllamaGenerateTransformation(
+            self.prompt_feature.name, self.model, self.system or "", self.host_env
+        )
 
 
 @dataclass
 class DateComponentFactory(TransformationFactory):
-
     component: str
     feature: FeatureFactory
 
@@ -255,69 +246,8 @@ class DateComponentFactory(TransformationFactory):
         return DCTransformation(self.feature.name, self.component)
 
 
-@dataclass
-class DifferanceBetweenFactory(TransformationFactory):
-
-    first_feature: FeatureFactory
-    second_feature: FeatureFactory
-
-    @property
-    def using_features(self) -> list[FeatureFactory]:
-        return [self.first_feature, self.second_feature]
-
-    def compile(self) -> Transformation:
-        from aligned.schemas.transformation import Subtraction
-
-        return Subtraction(self.first_feature.name, self.second_feature.name)
-
-
-@dataclass
-class AdditionBetweenFactory(TransformationFactory):
-
-    first_feature: FeatureFactory
-    second_feature: FeatureFactory | Any
-
-    @property
-    def using_features(self) -> list[FeatureFactory]:
-        if isinstance(self.second_feature, FeatureFactory):
-            return [self.first_feature, self.second_feature]
-        else:
-            return [self.first_feature]
-
-    def compile(self) -> Transformation:
-        from aligned.schemas.transformation import Addition, AdditionValue
-
-        if isinstance(self.second_feature, FeatureFactory):
-            return Addition(self.first_feature.name, self.second_feature.name)
-        else:
-            return AdditionValue(self.first_feature.name, LiteralValue.from_value(self.second_feature))
-
-
-@dataclass
-class PowerFactory(TransformationFactory):
-
-    first: FeatureFactory
-    second: FeatureFactory | Any
-
-    @property
-    def using_features(self) -> list[FeatureFactory]:
-        if isinstance(self.second, FeatureFactory):
-            return [self.first, self.second]
-        return [self.first]
-
-    def compile(self) -> Transformation:
-        from aligned.schemas.transformation import Power, PowerFeature
-
-        if isinstance(self.second, FeatureFactory):
-            return PowerFeature(self.first.name, self.second.name)
-        else:
-            value = LiteralValue.from_value(self.second)
-            return Power(self.first.name, value)
-
-
 @dataclass
 class TimeDifferanceFactory(TransformationFactory):
-
     first_feature: FeatureFactory
     second_feature: FeatureFactory
 
@@ -333,7 +263,6 @@ class TimeDifferanceFactory(TransformationFactory):
 
 @dataclass
 class LogTransformFactory(TransformationFactory):
-
     feature: FeatureFactory
 
     @property
@@ -348,7 +277,6 @@ class LogTransformFactory(TransformationFactory):
 
 @dataclass
 class ReplaceFactory(TransformationFactory):
-
     values: dict[str, str]
     source_feature: FeatureFactory
 
@@ -359,12 +287,12 @@ class ReplaceFactory(TransformationFactory):
     def compile(self) -> Transformation:
         from aligned.schemas.transformation import ReplaceStrings
 
-        return ReplaceStrings(self.source_feature.name, self.values)
+        values = list(self.values.items())
+        return ReplaceStrings(self.source_feature.name, values)
 
 
 @dataclass
 class ToNumericalFactory(TransformationFactory):
-
     from_feature: FeatureFactory
 
     @property
@@ -372,14 +300,15 @@ class ToNumericalFactory(TransformationFactory):
         return [self.from_feature]
 
     def compile(self) -> Transformation:
-        from aligned.schemas.transformation import ToNumerical as ToNumericalTransformation
+        from aligned.schemas.transformation import (
+            ToNumerical as ToNumericalTransformation,
+        )
 
         return ToNumericalTransformation(self.from_feature.name)
 
 
 @dataclass
 class IsInFactory(TransformationFactory):
-
     feature: FeatureFactory
     values: list
 
@@ -393,41 +322,8 @@ class IsInFactory(TransformationFactory):
         return IsInTransformation(self.values, self.feature.name)
 
 
-@dataclass
-class AndFactory(TransformationFactory):
-
-    first_feature: FeatureFactory
-    second_feature: FeatureFactory
-
-    @property
-    def using_features(self) -> list[FeatureFactory]:
-        return [self.first_feature, self.second_feature]
-
-    def compile(self) -> Transformation:
-        from aligned.schemas.transformation import And as AndTransformation
-
-        return AndTransformation(self.first_feature.name, self.second_feature.name)
-
-
-@dataclass
-class OrFactory(TransformationFactory):
-
-    first_feature: FeatureFactory
-    second_feature: FeatureFactory
-
-    @property
-    def using_features(self) -> list[FeatureFactory]:
-        return [self.first_feature, self.second_feature]
-
-    def compile(self) -> Transformation:
-        from aligned.schemas.transformation import Or as OrTransformation
-
-        return OrTransformation(self.first_feature.name, self.second_feature.name)
-
-
 @dataclass
 class InverseFactory(TransformationFactory):
-
     from_feature: FeatureFactory
 
     @property
@@ -455,27 +351,33 @@ class ConstantFillNaStrategy(FillNaStrategy):
 
 @dataclass
 class FillMissingFactory(TransformationFactory):
-
     feature: FeatureFactory
-    strategy: FillNaStrategy
+    value: LiteralValue | FeatureFactory
 
     @property
     def using_features(self) -> list[FeatureFactory]:
-        return [self.feature]
+        if isinstance(self.value, LiteralValue):
+            return [self.feature]
+        else:
+            return [self.feature, self.value]
 
     def compile(self) -> Transformation:
         from aligned.schemas.transformation import FillNaValues
 
-        fill_value = self.strategy.compile()
-
-        return FillNaValues(
-            key=self.feature.name, value=LiteralValue.from_value(fill_value), dtype=self.feature.dtype
-        )
+        if isinstance(self.value, LiteralValue):
+            return FillNaValues(
+                key=self.feature.name, value=self.value, dtype=self.feature.dtype
+            )
+        else:
+            return FillNaValuesColumns(
+                key=self.feature.name,
+                fill_key=self.value.name,
+                dtype=self.feature.dtype,
+            )
 
 
 @dataclass
 class FloorFactory(TransformationFactory):
-
     feature: FeatureFactory
 
     @property
@@ -490,7 +392,6 @@ class FloorFactory(TransformationFactory):
 
 @dataclass
 class CeilFactory(TransformationFactory):
-
     feature: FeatureFactory
 
     @property
@@ -505,7 +406,6 @@ class CeilFactory(TransformationFactory):
 
 @dataclass
 class RoundFactory(TransformationFactory):
-
     feature: FeatureFactory
 
     @property
@@ -515,12 +415,11 @@ class RoundFactory(TransformationFactory):
     def compile(self) -> Transformation:
         from aligned.schemas.transformation import Round
 
-        return Round(self.feature.name)
+        return Round(Expression.from_value(self.feature))
 
 
 @dataclass
 class AbsoluteFactory(TransformationFactory):
-
     feature: FeatureFactory
 
     @property
@@ -530,14 +429,72 @@ class AbsoluteFactory(TransformationFactory):
     def compile(self) -> Transformation:
         from aligned.schemas.transformation import Absolute
 
-        return Absolute(self.feature.name)
+        return Absolute(Expression.from_value(self.feature))
+
+
+@dataclass
+class MapRowTransformation(TransformationFactory):
+    dtype: FeatureFactory
+    method: (
+        Callable[[Any, dict, ContractStore], Any] | Callable[[dict, ContractStore], Any]
+    )
+    _using_features: list[FeatureFactory]
+
+    @property
+    def using_features(self) -> list[FeatureFactory]:
+        return self._using_features
+
+    def compile(self) -> Transformation:
+        import inspect
+        import types
+        import dill
+        from aligned.schemas.transformation import PolarsMapRowTransformation
+
+        if (
+            isinstance(self.method, types.LambdaType)
+            and self.method.__name__ == "<lambda>"
+        ):
+            raise NotImplementedError(type(self))
+
+        function_name = dill.source.getname(self.method)
+        assert isinstance(function_name, str), "Need a function name"
+        raw_code = inspect.getsource(self.method)
+
+        code = ""
+
+        indents: int | None = None
+        start_signature = f"def {function_name}"
+        strip_self = False
+
+        for line in raw_code.splitlines(keepends=True):
+            if strip_self:
+                code += line.replace("self,", "")
+                strip_self = False
+            elif start_signature in line:
+                stripped = line.lstrip()
+                indents = len(line) - len(stripped)
+                stripped = stripped.replace("self,", "")
+                code += stripped
+
+                if line.endswith("(\n"):
+                    strip_self = True
+            elif indents:
+                if len(line) > indents:
+                    code += line[:indents].lstrip() + line[indents:]
+                else:
+                    code += line
+
+        return PolarsMapRowTransformation(
+            code=code,
+            function_name=function_name,
+            dtype=self.dtype.dtype,
+        )
 
 
 @dataclass
 class PandasTransformationFactory(TransformationFactory):
-
     dtype: FeatureFactory
-    method: Callable[[pd.DataFrame], pd.Series]
+    method: Callable[[pd.DataFrame, ContractStore], pd.Series]
     _using_features: list[FeatureFactory]
 
     @property
@@ -550,27 +507,56 @@ class PandasTransformationFactory(TransformationFactory):
 
         import dill
 
-        from aligned.schemas.transformation import PandasFunctionTransformation, PandasLambdaTransformation
+        from aligned.schemas.transformation import (
+            PandasFunctionTransformation,
+            PandasLambdaTransformation,
+        )
 
-        if isinstance(self.method, types.LambdaType) and self.method.__name__ == '<lambda>':
+        if (
+            isinstance(self.method, types.LambdaType)
+            and self.method.__name__ == "<lambda>"
+        ):
             return PandasLambdaTransformation(
                 method=dill.dumps(self.method),
                 code=inspect.getsource(self.method).strip(),
                 dtype=self.dtype.dtype,
             )
         else:
+            function_name = dill.source.getname(self.method)
+            assert isinstance(function_name, str), "Need a function name"
+            raw_code = inspect.getsource(self.method)
+
+            code = ""
+
+            indents: int | None = None
+            start_signature = f"def {function_name}"
+
+            for line in raw_code.splitlines(keepends=True):
+                if indents:
+                    if len(line) > indents:
+                        code += line[:indents].lstrip() + line[indents:]
+                    else:
+                        code += line
+
+                if start_signature in line:
+                    stripped = line.lstrip()
+                    indents = len(line) - len(stripped)
+                    stripped = stripped.replace(
+                        f"{start_signature}(self,", f"{start_signature}("
+                    )
+                    code += stripped
+
             return PandasFunctionTransformation(
-                code=inspect.getsource(self.method),
-                function_name=dill.source.getname(self.method),
+                code=code,
+                function_name=function_name,
                 dtype=self.dtype.dtype,
             )
 
 
 @dataclass
 class PolarsTransformationFactory(TransformationFactory):
-
     dtype: FeatureFactory
-    method: pl.Expr | Callable[[pl.LazyFrame, pl.Expr], pl.LazyFrame]
+    method: pl.Expr | Callable[[pl.LazyFrame, pl.Expr, ContractStore], pl.LazyFrame]
     _using_features: list[FeatureFactory]
 
     @property
@@ -583,39 +569,73 @@ class PolarsTransformationFactory(TransformationFactory):
 
         import dill
 
-        from aligned.schemas.transformation import PolarsFunctionTransformation, PolarsLambdaTransformation
+        from aligned.schemas.transformation import (
+            PolarsFunctionTransformation,
+            PolarsLambdaTransformation,
+            PolarsExpression,
+        )
 
         if isinstance(self.method, pl.Expr):
-            code = str(self.method)
-            return PolarsLambdaTransformation(
-                method=dill.dumps(self.method), code=code.strip(), dtype=self.dtype.dtype
+            return PolarsExpression(
+                self.method.meta.serialize(format="json"), dtype=self.dtype.dtype
             )
         else:
-            code = inspect.getsource(self.method)
+            function_name = dill.source.getname(self.method)
+            assert isinstance(function_name, str), "Need a function name"
+            raw_code = inspect.getsource(self.method)
 
-        if isinstance(self.method, types.LambdaType) and self.method.__name__ == '<lambda>':
+            code = ""
+
+            indents: int | None = None
+            strip_self = False
+            start_signature = f"def {function_name}"
+
+            for line in raw_code.splitlines(keepends=True):
+                if strip_self:
+                    code += line.replace("self,", "")
+                    strip_self = False
+                elif start_signature in line:
+                    stripped = line.lstrip()
+                    indents = len(line) - len(stripped)
+                    stripped = stripped.replace("self,", "")
+                    code += stripped
+
+                    if line.endswith("(\n"):
+                        strip_self = True
+                elif indents:
+                    if len(line) > indents:
+                        code += line[:indents].lstrip() + line[indents:]
+                    else:
+                        code += line
+
+        if (
+            isinstance(self.method, types.LambdaType)
+            and self.method.__name__ == "<lambda>"
+        ):
             return PolarsLambdaTransformation(
-                method=dill.dumps(self.method), code=code.strip(), dtype=self.dtype.dtype
+                method=dill.dumps(self.method),
+                code=code.strip(),
+                dtype=self.dtype.dtype,
             )
         else:
+            function_name = dill.source.getname(self.method)
+            assert isinstance(
+                function_name, str
+            ), f"Expected string got {type(function_name)}"
             return PolarsFunctionTransformation(
                 code=code,
-                function_name=dill.source.getname(self.method),
+                function_name=function_name,
                 dtype=self.dtype.dtype,
             )
 
 
 class AggregatableTransformation:
-
-    group_by: list[FeatureFactory] | None = field(default=None)
-
-    def copy(self) -> 'AggregatableTransformation':
-        pass
+    def copy(self) -> "AggregatableTransformation":
+        raise NotImplementedError(type(self))
 
 
 @dataclass
 class MeanTransfomrationFactory(TransformationFactory, AggregatableTransformation):
-
     feature: FeatureFactory
     over: timedelta | None = field(default=None)
     group_by: list[FeatureFactory] | None = field(default=None)
@@ -628,21 +648,18 @@ class MeanTransfomrationFactory(TransformationFactory, AggregatableTransformatio
             return [self.feature]
 
     def compile(self) -> Transformation:
-        from aligned.schemas.transformation import Mean
+        from aligned.schemas.transformation import MeanAggregation
 
-        return Mean(
-            key=self.feature.name, group_keys=[feat.name for feat in self.group_by] if self.group_by else None
-        )
+        return MeanAggregation(key=self.feature.name)
 
-    def copy(self) -> 'MeanTransfomrationFactory':
+    def copy(self) -> "MeanTransfomrationFactory":
         return MeanTransfomrationFactory(self.feature, self.over, self.group_by)
 
 
 @dataclass
 class WordVectoriserFactory(TransformationFactory):
-
     feature: FeatureFactory
-    model: TextVectoriserModel
+    model: EmbeddingModel
 
     @property
     def using_features(self) -> list[FeatureFactory]:
@@ -655,8 +672,21 @@ class WordVectoriserFactory(TransformationFactory):
 
 
 @dataclass
-class LoadImageFactory(TransformationFactory):
+class LoadImageBytesFactory(TransformationFactory):
+    url_feature: FeatureFactory
 
+    @property
+    def using_features(self) -> list[FeatureFactory]:
+        return [self.url_feature]
+
+    def compile(self) -> Transformation:
+        from aligned.schemas.transformation import LoadImageUrlBytes
+
+        return LoadImageUrlBytes(self.url_feature.name)
+
+
+@dataclass
+class LoadImageFactory(TransformationFactory):
     url_feature: FeatureFactory
 
     @property
@@ -671,7 +701,6 @@ class LoadImageFactory(TransformationFactory):
 
 @dataclass
 class GrayscaleImageFactory(TransformationFactory):
-
     image_feature: FeatureFactory
 
     @property
@@ -686,10 +715,9 @@ class GrayscaleImageFactory(TransformationFactory):
 
 @dataclass
 class AppendStrings(TransformationFactory):
-
     first_feature: FeatureFactory
     second_feature: FeatureFactory | LiteralValue
-    separator: str = field(default='')
+    separator: str = field(default="")
 
     @property
     def using_features(self) -> list[FeatureFactory]:
@@ -702,14 +730,32 @@ class AppendStrings(TransformationFactory):
         from aligned.schemas.transformation import AppendConstString, AppendStrings
 
         if isinstance(self.second_feature, LiteralValue):
-            return AppendConstString(self.first_feature.name, self.second_feature.value)
+            return AppendConstString(
+                self.first_feature.name, self.second_feature.python_value
+            )
         else:
-            return AppendStrings(self.first_feature.name, self.second_feature.name, self.separator)
+            return AppendStrings(
+                self.first_feature.name, self.second_feature.name, self.separator
+            )
+
+
+@dataclass
+class StructFieldFactory(TransformationFactory):
+    struct_feature: FeatureFactory
+    field_name: str
+
+    @property
+    def using_features(self) -> list[FeatureFactory]:
+        return [self.struct_feature]
+
+    def compile(self) -> Transformation:
+        from aligned.schemas.transformation import StructField
+
+        return StructField(self.struct_feature.name, self.field_name)
 
 
 @dataclass
 class JsonPathFactory(TransformationFactory):
-
     json_feature: FeatureFactory
     path: str
 
@@ -725,7 +771,6 @@ class JsonPathFactory(TransformationFactory):
 
 @dataclass
 class PresignedAwsUrlFactory(TransformationFactory):
-
     aws_config: AwsS3Config
     url_feature: FeatureFactory
     max_age_seconds: int = field(default=30)
@@ -737,15 +782,16 @@ class PresignedAwsUrlFactory(TransformationFactory):
     def compile(self) -> Transformation:
         from aligned.schemas.transformation import PresignedAwsUrl
 
-        return PresignedAwsUrl(self.aws_config, self.url_feature.name, self.max_age_seconds)
+        return PresignedAwsUrl(
+            self.aws_config, self.url_feature.name, self.max_age_seconds
+        )
 
 
 @dataclass
 class PrependConstString(TransformationFactory):
-
     first_feature: str
     second_feature: FeatureFactory
-    separator: str = field(default='')
+    separator: str = field(default="")
 
     @property
     def using_features(self) -> list[FeatureFactory]:
@@ -759,7 +805,6 @@ class PrependConstString(TransformationFactory):
 
 @dataclass
 class ClipFactory(TransformationFactory):
-
     feature: FeatureFactory
     lower_bound: int | float
     upper_bound: int | float
@@ -772,29 +817,80 @@ class ClipFactory(TransformationFactory):
         from aligned.schemas.transformation import Clip
 
         return Clip(
-            self.feature.name,
+            Expression.from_value(self.feature),
             LiteralValue.from_value(self.lower_bound),
             LiteralValue.from_value(self.upper_bound),
         )
 
 
 @dataclass
-class MultiplyFactory(TransformationFactory):
-
-    first: FeatureFactory
-    behind: FeatureFactory | LiteralValue
+class LoadFeature(TransformationFactory):
+    entities: dict[str, FeatureFactory]
+    feature: FeatureReference
+    dtype: FeatureType
 
     @property
     def using_features(self) -> list[FeatureFactory]:
-        if isinstance(self.behind, LiteralValue):
-            return [self.first]
-        else:
-            return [self.first, self.behind]
+        return list(self.entities.values())
 
     def compile(self) -> Transformation:
-        from aligned.schemas.transformation import Multiply, MultiplyValue
+        from aligned.compiler.feature_factory import List
+        from aligned.schemas.transformation import LoadFeature
 
-        if isinstance(self.behind, LiteralValue):
-            return MultiplyValue(self.first.name, self.behind)
-        else:
-            return Multiply(self.first.name, self.behind.name)
+        explode_key: str | None = None
+        for feature in self.entities.values():
+            if isinstance(feature, List):
+                explode_key = feature.name
+
+        return LoadFeature(
+            {key: value.name for key, value in self.entities.items()},
+            self.feature,
+            explode_key,
+            dtype=self.dtype,
+        )
+
+
+@dataclass
+class FormatString(TransformationFactory):
+    format: str
+    features: list[FeatureFactory]
+
+    @property
+    def using_features(self) -> list[FeatureFactory]:
+        return self.features
+
+    def compile(self) -> Transformation:
+        from aligned.schemas.transformation import FormatStringTransformation
+
+        return FormatStringTransformation(
+            self.format, [feature.name for feature in self.features]
+        )
+
+
+@dataclass
+class ListDotProduct(TransformationFactory):
+    left: FeatureFactory
+    right: FeatureFactory
+
+    @property
+    def using_features(self) -> list[FeatureFactory]:
+        return [self.left, self.right]
+
+    def compile(self) -> Transformation:
+        from aligned.schemas.transformation import ListDotProduct
+
+        return ListDotProduct(self.left.name, self.right.name)
+
+
+@dataclass
+class HashColumns(TransformationFactory):
+    columns: list[FeatureFactory]
+
+    @property
+    def using_features(self) -> list[FeatureFactory]:
+        return self.columns
+
+    def compile(self) -> Transformation:
+        from aligned.schemas.transformation import HashColumns
+
+        return HashColumns([col.name for col in self.columns])
