@@ -3,14 +3,18 @@ from __future__ import annotations
 import logging
 from dataclasses import dataclass, field
 
-import pandas as pd
 import polars as pl
 
+from aligned.lazy_imports import pandas as pd
 from aligned.psql.jobs import PostgreSqlJob
 from aligned.redshift.sql_job import SqlColumn, TableFetch
-from aligned.request.retrival_request import RequestResult, RetrivalRequest
-from aligned.retrival_job import FactualRetrivalJob, RetrivalJob
-from aligned.schemas.derivied_feature import AggregatedFeature, AggregateOver, DerivedFeature
+from aligned.request.retrieval_request import RequestResult, RetrievalRequest
+from aligned.retrieval_job import RetrievalJob
+from aligned.schemas.derivied_feature import (
+    AggregatedFeature,
+    AggregateOver,
+    DerivedFeature,
+)
 from aligned.schemas.feature import FeatureLocation, FeatureType
 from aligned.schemas.transformation import RedshiftTransformation
 from aligned.sources.redshift import RedshiftSQLConfig, RedshiftSQLDataSource
@@ -28,11 +32,11 @@ class SqlValue:
         if self.value:
             return f"'{self.value}'::{self.data_type}"
         else:
-            return f'NULL::{self.data_type}'
+            return f"NULL::{self.data_type}"
 
 
 @dataclass
-class FactRedshiftJob(FactualRetrivalJob):
+class FactRedshiftJob(RetrievalJob):
     """Fetches features for defined facts within a postgres DB
 
     It is supported to fetch from different tables, in one request
@@ -42,17 +46,17 @@ class FactRedshiftJob(FactualRetrivalJob):
     """
 
     sources: dict[FeatureLocation, RedshiftSQLDataSource]
-    requests: list[RetrivalRequest]
-    facts: RetrivalJob
+    requests: list[RetrievalRequest]
+    facts: RetrievalJob
 
-    entity_table_name: str = field(default='entities')
+    entity_table_name: str = field(default="entities")
 
     @property
     def request_result(self) -> RequestResult:
         return RequestResult.from_request_list(self.requests)
 
     @property
-    def retrival_requests(self) -> list[RetrivalRequest]:
+    def retrieval_requests(self) -> list[RetrievalRequest]:
         return self.requests
 
     @property
@@ -63,41 +67,48 @@ class FactRedshiftJob(FactualRetrivalJob):
         job = await self.psql_job()
         return await job.to_pandas()
 
-    async def to_polars(self) -> pl.LazyFrame:
+    async def to_lazy_polars(self) -> pl.LazyFrame:
         job = await self.psql_job()
-        return await job.to_polars()
+        return await job.to_lazy_polars()
 
     async def psql_job(self) -> PostgreSqlJob:
         if isinstance(self.facts, PostgreSqlJob):
-            return PostgreSqlJob(self.config.psql_config, self.build_sql_entity_query(self.facts))
-        raise ValueError(f'Redshift only support SQL entity queries. Got: {self.facts}')
+            return PostgreSqlJob(
+                self.config.psql_config, self.build_sql_entity_query(self.facts)
+            )
+        raise ValueError(f"Redshift only support SQL entity queries. Got: {self.facts}")
 
     def describe(self) -> str:
         if isinstance(self.facts, PostgreSqlJob):
-            return PostgreSqlJob(self.config.psql_config, self.build_sql_entity_query(self.facts)).describe()
-        raise ValueError(f'Redshift only support SQL entity queries. Got: {self.facts}')
+            return PostgreSqlJob(
+                self.config.psql_config, self.build_sql_entity_query(self.facts)
+            ).describe()
+        raise ValueError(f"Redshift only support SQL entity queries. Got: {self.facts}")
 
     def dtype_to_sql_type(self, dtype: object) -> str:
         if isinstance(dtype, str):
             return dtype
-        if dtype == FeatureType('').string:
-            return 'text'
-        if dtype == FeatureType('').uuid:
-            return 'uuid'
-        if dtype == FeatureType('').int32 or dtype == FeatureType('').int64:
-            return 'integer'
-        if dtype == FeatureType('').datetime:
-            return 'TIMESTAMP WITH TIME ZONE'
-        return 'uuid'
-
-    def value_selection(self, request: RetrivalRequest, entities_has_event_timestamp: bool) -> TableFetch:
+        if dtype == FeatureType.string():
+            return "text"
+        if dtype == FeatureType.uuid():
+            return "uuid"
+        if dtype == FeatureType.int32() or dtype == FeatureType.int64():
+            return "integer"
+        if dtype == FeatureType.datetime():
+            return "TIMESTAMP WITH TIME ZONE"
+        return "uuid"
 
+    def value_selection(
+        self, request: RetrievalRequest, entities_has_event_timestamp: bool
+    ) -> TableFetch:
         source = self.sources[request.location]
 
-        entity_selects = {f'{self.entity_table_name}.{entity}' for entity in request.entity_names}
+        entity_selects = {
+            f"{self.entity_table_name}.{entity}" for entity in request.entity_names
+        }
         field_selects = list(
             request.all_required_feature_names.union(entity_selects).union(
-                {f'{self.entity_table_name}.row_id'}
+                {f"{self.entity_table_name}.row_id"}
             )
         )
         field_identifiers = source.feature_identifier_for(field_selects)
@@ -109,15 +120,15 @@ class FactRedshiftJob(FactualRetrivalJob):
 
         entities = list(request.entity_names)
         entity_db_name = source.feature_identifier_for(entities)
-        sort_query = f'{self.entity_table_name}.row_id'
+        sort_query = f"{self.entity_table_name}.row_id"
 
         event_timestamp_clause: str | None = None
         if request.event_timestamp and entities_has_event_timestamp:
-            event_timestamp_column = source.feature_identifier_for([request.event_timestamp.name])[0]
-            event_timestamp_clause = (
-                f'{self.entity_table_name}.event_timestamp >= ta.{event_timestamp_column}'
-            )
-            sort_query += f', {event_timestamp_column} DESC'
+            event_timestamp_column = source.feature_identifier_for(
+                [request.event_timestamp.name]
+            )[0]
+            event_timestamp_clause = f"{self.entity_table_name}.event_timestamp >= ta.{event_timestamp_column}"
+            sort_query += f", {event_timestamp_column} DESC"
 
         join_conditions = [
             f'ta."{entity_db_name}" = {self.entity_table_name}.{entity}'
@@ -132,8 +143,10 @@ class FactRedshiftJob(FactualRetrivalJob):
                 if feature_name not in request.all_feature_names:
                     continue
                 selects.add(SqlColumn(feature_name, feature_name))
-                table_id = f'{feature_name}_list'
-                source_id_column = reference.join_column or list(request.entity_names)[0]
+                table_id = f"{feature_name}_list"
+                source_id_column = (
+                    reference.join_column or list(request.entity_names)[0]
+                )
                 column = f"'[\"' || listagg({reference.value_column}, '\",\"') || '\"]'"
                 join_table = TableFetch(
                     name=table_id,
@@ -146,12 +159,14 @@ class FactRedshiftJob(FactualRetrivalJob):
                     },
                     group_by=[reference.id_column],
                 )
-                table_join_condition = f'{table_id}.{reference.id_column} = ta.{source_id_column}'
+                table_join_condition = (
+                    f"{table_id}.{reference.id_column} = ta.{source_id_column}"
+                )
                 join_tables.append((join_table, table_join_condition))
 
         rename_fetch = TableFetch(
-            name=f'{request.name}_cte',
-            id_column='row_id',
+            name=f"{request.name}_cte",
+            id_column="row_id",
             table=source.table,
             columns=selects,
             joins=join_conditions,
@@ -168,12 +183,14 @@ class FactRedshiftJob(FactualRetrivalJob):
             and all([name not in derived_map for name in feature.depending_on_names])
         ]
         if derived_features:
-            derived_alias = source.feature_identifier_for([feature.name for feature in derived_features])
+            derived_alias = source.feature_identifier_for(
+                [feature.name for feature in derived_features]
+            )
             derived_selects = {
                 SqlColumn(feature.transformation.as_redshift(), name)
                 for feature, name in zip(derived_features, derived_alias)
                 if isinstance(feature.transformation, RedshiftTransformation)
-            }.union({SqlColumn('*', '*')})
+            }.union({SqlColumn("*", "*")})
 
             return TableFetch(
                 name=rename_fetch.name,
@@ -185,25 +202,30 @@ class FactRedshiftJob(FactualRetrivalJob):
             return rename_fetch
 
     def sql_aggregated_request(
-        self, window: AggregateOver, features: set[AggregatedFeature], request: RetrivalRequest
+        self,
+        window: AggregateOver,
+        features: set[AggregatedFeature],
+        request: RetrievalRequest,
     ) -> TableFetch:
         source = self.sources[request.location]
-        name = f'{request.name}_agg_cte'
+        name = f"{request.name}_agg_cte"
 
         if not all(
             [
-                isinstance(feature.derived_feature.transformation, RedshiftTransformation)
+                isinstance(
+                    feature.derived_feature.transformation, RedshiftTransformation
+                )
                 for feature in features
             ]
         ):
-            raise ValueError('All features must have a RedshiftTransformation')
+            raise ValueError("All features must have a RedshiftTransformation")
 
         group_by_names = {feature.name for feature in window.group_by}
         if window.window:
             time_window_config = window.window
             time_window = int(time_window_config.time_window.total_seconds())
-            name = f'{request.name}_agg_{time_window}_cte'
-            group_by_names = {f'{self.entity_table_name}s.row_id'}
+            name = f"{request.name}_agg_{time_window}_cte"
+            group_by_names = {f"{self.entity_table_name}s.row_id"}
 
         aggregates = {
             SqlColumn(
@@ -211,26 +233,29 @@ class FactRedshiftJob(FactualRetrivalJob):
                 feature.name,
             )
             for feature in features
+            if isinstance(
+                feature.derived_feature.transformation, RedshiftTransformation
+            )
         }
 
         id_column = window.group_by[0].name
         event_timestamp_clause: str | None = None
         if request.event_timestamp:
-            id_column = 'row_id'
-            group_by_names = {f'{self.entity_table_name}.row_id'}
+            id_column = "row_id"
+            group_by_names = {f"{self.entity_table_name}.row_id"}
             # Use row_id as the main join key
-            event_timestamp_name = source.feature_identifier_for([request.event_timestamp.name])[0]
+            event_timestamp_name = source.feature_identifier_for(
+                [request.event_timestamp.name]
+            )[0]
             if window.window:
                 time_window_config = window.window
                 window_in_seconds = int(time_window_config.time_window.total_seconds())
                 event_timestamp_clause = (
-                    f'ta.{event_timestamp_name} BETWEEN {self.entity_table_name}.event_timestamp'
+                    f"ta.{event_timestamp_name} BETWEEN {self.entity_table_name}.event_timestamp"
                     f" - interval '{window_in_seconds} seconds' AND {self.entity_table_name}.event_timestamp"
                 )
             else:
-                event_timestamp_clause = (
-                    f'ta.{event_timestamp_name} <= {self.entity_table_name}.event_timestamp'
-                )
+                event_timestamp_clause = f"ta.{event_timestamp_name} <= {self.entity_table_name}.event_timestamp"
 
         group_by_selects = {SqlColumn(feature, feature) for feature in group_by_names}
 
@@ -243,7 +268,9 @@ class FactRedshiftJob(FactualRetrivalJob):
         if event_timestamp_clause:
             join_conditions.append(event_timestamp_clause)
 
-        field_selects = request.all_required_feature_names.union({f'{self.entity_table_name}.*'})
+        field_selects = request.all_required_feature_names.union(
+            {f"{self.entity_table_name}.*"}
+        )
         field_identifiers = source.feature_identifier_for(field_selects)
         selects = {
             SqlColumn(db_field_name, feature)
@@ -251,8 +278,8 @@ class FactRedshiftJob(FactualRetrivalJob):
         }
 
         rename_table = TableFetch(
-            name='ta',
-            id_column='row_id',
+            name="ta",
+            id_column="row_id",
             table=source.table,
             columns=selects,
             joins=join_conditions,
@@ -280,10 +307,12 @@ class FactRedshiftJob(FactualRetrivalJob):
                     continue
                 sub_feature = derived_map[depended]
                 if depended.isnumeric():
-                    assert isinstance(sub_feature.transformation, RedshiftTransformation)
+                    assert isinstance(
+                        sub_feature.transformation, RedshiftTransformation
+                    )
                     hidden_sql = sub_feature.transformation.as_redshift()
                     hidden_sql = resolve_sql_transformation(sub_feature.name)
-                    sql_query = sql_query.replace(depended, f'({hidden_sql})')
+                    sql_query = sql_query.replace(depended, f"({hidden_sql})")
 
             return sql_query
 
@@ -293,17 +322,17 @@ class FactRedshiftJob(FactualRetrivalJob):
                     needed_derived_features.add(derived_map[depended])
 
         if needed_derived_features:
-            features = {
+            sql_features = {
                 SqlColumn(resolve_sql_transformation(feature.name), feature.name)
                 for feature in needed_derived_features
             }.union(
-                {SqlColumn('*', '*')}
+                {SqlColumn("*", "*")}
             )  # Need the * in order to select the "original values"
             select_table = TableFetch(
-                name='derived',
+                name="derived",
                 id_column=rename_table.id_column,
                 table=rename_table,
-                columns=features,
+                columns=sql_features,
             )
         else:
             select_table = rename_table
@@ -318,8 +347,9 @@ class FactRedshiftJob(FactualRetrivalJob):
             group_by=list(group_by_names),
         )
 
-    def aggregated_values_from_request(self, request: RetrivalRequest) -> list[TableFetch]:
-
+    def aggregated_values_from_request(
+        self, request: RetrievalRequest
+    ) -> list[TableFetch]:
         aggregation_windows: dict[AggregateOver, set[AggregatedFeature]] = {}
 
         for aggregate in request.aggregated_features:
@@ -329,7 +359,9 @@ class FactRedshiftJob(FactualRetrivalJob):
                 aggregation_windows[aggregate.aggregate_over].add(aggregate)
 
         fetches: list[TableFetch] = []
-        supported_aggregation_features = set(request.feature_names).union(request.entity_names)
+        supported_aggregation_features = set(request.feature_names).union(
+            request.entity_names
+        )
         for feature in request.derived_features:
             if isinstance(feature.transformation, RedshiftTransformation):
                 supported_aggregation_features.add(feature.name)
@@ -346,35 +378,36 @@ class FactRedshiftJob(FactualRetrivalJob):
                 fetches.append(self.sql_aggregated_request(window, aggregates, request))
             else:
                 raise ValueError(
-                    f'Only SQL aggregates are supported at the moment. Missing features {missing_features}'
+                    f"Only SQL aggregates are supported at the moment. Missing features {missing_features}"
                 )
 
         return fetches
 
     def build_sql_entity_query(self, sql_facts: PostgreSqlJob) -> str:
-
         final_select_names: set[str] = set()
         has_event_timestamp = False
         all_entities = set()
 
-        if 'event_timestamp' in sql_facts.query:
+        if "event_timestamp" in sql_facts.query:
             has_event_timestamp = True
-            all_entities.add('event_timestamp')
+            all_entities.add("event_timestamp")
 
         for request in self.requests:
             final_select_names = final_select_names.union(
-                {f'{self.entity_table_name}.{entity}' for entity in request.entity_names}
+                {
+                    f"{self.entity_table_name}.{entity}"
+                    for entity in request.entity_names
+                }
             )
 
         if has_event_timestamp:
-            final_select_names.add('event_timestamp')
+            final_select_names.add("event_timestamp")
 
         # Add the joins to the fact
 
         tables: list[TableFetch] = []
         aggregates: list[TableFetch] = []
         for request in self.requests:
-
             all_entities.update(request.entity_names)
 
             if request.aggregated_features:
@@ -383,9 +416,9 @@ class FactRedshiftJob(FactualRetrivalJob):
 
                 for aggregate in aggregate_fetches:
                     agg_features = {
-                        f'{aggregate.name}.{column.alias}'
+                        f"{aggregate.name}.{column.alias}"
                         for column in aggregate.columns
-                        if column.alias != f'{self.entity_table_name}.row_id'
+                        if column.alias != f"{self.entity_table_name}.row_id"
                         and column.alias not in all_entities
                     }
                     final_select_names = final_select_names.union(agg_features)
@@ -393,29 +426,32 @@ class FactRedshiftJob(FactualRetrivalJob):
                 fetch = self.value_selection(request, has_event_timestamp)
                 tables.append(fetch)
                 final_select_names = final_select_names.union(
-                    {f'{fetch.name}.{feature}' for feature in request.all_required_feature_names}
+                    {
+                        f"{fetch.name}.{feature}"
+                        for feature in request.all_required_feature_names
+                    }
                 )
 
         all_entities_list = list(all_entities)
-        all_entities_str = ', '.join(all_entities_list)
-        all_entities_list.append('row_id')
+        all_entities_str = ", ".join(all_entities_list)
+        all_entities_list.append("row_id")
 
         entity_query = (
-            f'SELECT {all_entities_str}, ROW_NUMBER() OVER ()'
-            f'AS row_id FROM ({sql_facts.query}) AS {self.entity_table_name}'
+            f"SELECT {all_entities_str}, ROW_NUMBER() OVER ()"
+            f"AS row_id FROM ({sql_facts.query}) AS {self.entity_table_name}"
         )
-        joins = '\n    '.join(
+        joins = "\n    ".join(
             [
-                f'LEFT JOIN {table.name} ON {table.name}.row_id = {self.entity_table_name}.row_id'
+                f"LEFT JOIN {table.name} ON {table.name}.row_id = {self.entity_table_name}.row_id"
                 for table in tables
             ]
         )
         if aggregates:
-            joins += '\n    '
-            joins += '\n    '.join(
+            joins += "\n    "
+            joins += "\n    ".join(
                 [
-                    f'LEFT JOIN {table.name} ON '
-                    f'{table.name}.{table.id_column} = {self.entity_table_name}.{table.id_column}'
+                    f"LEFT JOIN {table.name} ON "
+                    f"{table.name}.{table.id_column} = {self.entity_table_name}.{table.id_column}"
                     for table in aggregates
                 ]
             )
@@ -427,7 +463,7 @@ class FactRedshiftJob(FactualRetrivalJob):
             aggregates=aggregates,
             final_select=list(final_select_names),
             final_joins=joins,
-            event_timestamp_col='event_timestamp' if has_event_timestamp else None,
+            event_timestamp_col="event_timestamp" if has_event_timestamp else None,
         )
 
     def generate_query(
@@ -440,7 +476,6 @@ class FactRedshiftJob(FactualRetrivalJob):
         final_joins: str,
         event_timestamp_col: str | None,
     ) -> str:
-
         query = f"""
 WITH {self.entity_table_name} (
     { ', '.join(entity_columns) }
@@ -464,7 +499,6 @@ WITH {self.entity_table_name} (
 
         query = query[:-1]  # Dropping the last comma
         if event_timestamp_col:
-
             query += f"""
 SELECT val.*
 FROM (
