@@ -4,41 +4,50 @@ from dataclasses import dataclass, field
 from datetime import timedelta
 
 from aligned.schemas.codable import Codable
-from aligned.schemas.feature import Constraint, Feature, FeatureLocation, FeatureReferance, FeatureType
+from aligned.schemas.feature import (
+    Constraint,
+    Feature,
+    FeatureLocation,
+    FeatureReference,
+    FeatureType,
+)
 from aligned.schemas.transformation import Transformation
 
 
 class DerivedFeature(Feature):
-
-    depending_on: set[FeatureReferance]
+    depending_on: set[FeatureReference]
     transformation: Transformation
+    loads_feature: FeatureReference | None = None
     depth: int = 1
 
     def __init__(
         self,
         name: str,
         dtype: FeatureType,
-        depending_on: set[FeatureReferance],
+        depending_on: set[FeatureReference],
         transformation: Transformation,
         depth: int,
         description: str | None = None,
-        tags: dict[str, str] | None = None,
+        tags: list[str] | None = None,
         constraints: set[Constraint] | None = None,
+        loads_feature: FeatureReference | None = None,
     ):
         self.name = name
+        self.tags = tags
+        self.depth = depth
         self.dtype = dtype
+        self.description = description
+        self.constraints = constraints
         self.depending_on = depending_on
         self.transformation = transformation
-        self.depth = depth
-        self.description = description
-        self.tags = tags
-        self.constraints = constraints
+        self.loads_feature = loads_feature
+        self.default_value = None
 
     def __pre_serialize__(self) -> DerivedFeature:
         from aligned.schemas.transformation import SupportedTransformations
 
         for feature in self.depending_on:
-            assert isinstance(feature, FeatureReferance)
+            assert isinstance(feature, FeatureReference)
 
         assert isinstance(self.transformation, Transformation)
         assert self.transformation.name in SupportedTransformations.shared().types
@@ -67,11 +76,10 @@ class DerivedFeature(Feature):
 @dataclass
 class AggregationTimeWindow(Codable):
     time_window: timedelta
-    time_column: FeatureReferance
+    time_column: FeatureReference
 
-    @property
-    def group_by_names(self) -> list[str]:
-        return [feature.name for feature in self.group_by]
+    every_interval: timedelta | None = field(default=None)
+    offset_interval: timedelta | None = field(default=None)
 
     def __hash__(self) -> int:
         return self.time_window.__hash__()
@@ -79,7 +87,7 @@ class AggregationTimeWindow(Codable):
 
 @dataclass
 class AggregateOver(Codable):
-    group_by: list[FeatureReferance]
+    group_by: list[FeatureReference]
     window: AggregationTimeWindow | None = field(default=None)
     condition: DerivedFeature | None = field(default=None)
 
@@ -91,7 +99,7 @@ class AggregateOver(Codable):
         if self.window:
             return self.window.__hash__()
 
-        name = ''
+        name = ""
         for feature in self.group_by:
             name += feature.name
         return name.__hash__()
@@ -99,7 +107,6 @@ class AggregateOver(Codable):
 
 @dataclass
 class AggregatedFeature(Codable):
-
     derived_feature: DerivedFeature
     aggregate_over: AggregateOver
 
@@ -107,7 +114,7 @@ class AggregatedFeature(Codable):
         return self.derived_feature.name.__hash__()
 
     @property
-    def depending_on(self) -> set[FeatureReferance]:
+    def depending_on(self) -> set[FeatureReference]:
         return self.derived_feature.depending_on
 
     @property
