@@ -7,10 +7,9 @@ from datetime import datetime
 from pathlib import Path
 from typing import TYPE_CHECKING
 
-from aligned.enricher import Enricher
 from aligned.feature_source import FeatureSource, FeatureSourceFactory
 from aligned.schemas.codable import Codable
-from aligned.schemas.feature_view import CompiledCombinedFeatureView, CompiledFeatureView
+from aligned.schemas.feature_view import CompiledFeatureView
 from aligned.schemas.model import Model
 
 if TYPE_CHECKING:
@@ -51,24 +50,14 @@ class RepoReference:
         return self.repo_paths.get(self.selected)
 
     def feature_server(self, source: FeatureSource) -> FastAPI | FeatureSource:
-        import os
-
-        if os.environ.get('ALADDIN_ENABLE_SERVER', 'False').lower() == 'false':
-            return source
-
         from aligned.server import FastAPIServer
 
         if not (selected_file := self.selected_file):
-            raise ValueError('No selected file to serve features from')
-
-        try:
-            feature_store = asyncio.get_event_loop().run_until_complete(selected_file.feature_store())
-        except RuntimeError:
-            import nest_asyncio
-
-            nest_asyncio.apply()
-            feature_store = asyncio.new_event_loop().run_until_complete(selected_file.feature_store())
+            raise ValueError("No selected file to serve features from")
 
+        feature_store = asyncio.get_event_loop().run_until_complete(
+            selected_file.feature_store()
+        )
         return FastAPIServer.app(feature_store)
 
     @staticmethod
@@ -82,18 +71,19 @@ class RepoReference:
             module = import_module(module_path)
             obj = getattr(module, object)
             if isinstance(obj, StorageFileReference):
-                return RepoReference(env_var_name='const', repo_paths={'const': obj})
-            raise ValueError('No reference found')
+                return RepoReference(env_var_name="const", repo_paths={"const": obj})
+            raise ValueError("No reference found")
         except AttributeError:
-            raise ValueError('No reference found')
+            raise ValueError("No reference found")
         except ModuleNotFoundError:
-            raise ValueError('No reference found')
+            raise ValueError("No reference found")
 
 
 class FeatureServer:
     @staticmethod
     def from_reference(
-        reference: StorageFileReference, online_source: FeatureSource | FeatureSourceFactory | None = None
+        reference: StorageFileReference,
+        online_source: FeatureSource | FeatureSourceFactory | None = None,
     ) -> FastAPI | None:
         """Creates a feature server
         This can process and serve features for both models and feature views
@@ -116,60 +106,36 @@ class FeatureServer:
         Returns:
             FastAPI: A FastAPI instance that contains paths for fetching features
         """
-        import os
-
-        if os.environ.get('ALADDIN_ENABLE_SERVER', 'False').lower() == 'false':
-            return None
-
         from aligned.server import FastAPIServer
 
-        try:
-            feature_store = asyncio.get_event_loop().run_until_complete(reference.feature_store())
-        except RuntimeError:
-            import nest_asyncio
-
-            nest_asyncio.apply()
-            feature_store = asyncio.new_event_loop().run_until_complete(reference.feature_store())
-
+        feature_store = asyncio.get_event_loop().run_until_complete(
+            reference.feature_store()
+        )
         return FastAPIServer.app(feature_store.with_source(online_source))
 
 
-@dataclass
-class EnricherReference(Codable):
-    module: str
-    attribute_name: str
-    enricher: Enricher
-
-
 @dataclass
 class RepoMetadata(Codable):
     created_at: datetime
     name: str
+    repo_url: str | None = field(default=None)
     github_url: str | None = field(default=None)
 
 
 @dataclass
 class RepoDefinition(Codable):
-
     metadata: RepoMetadata
 
     feature_views: set[CompiledFeatureView] = field(default_factory=set)
-    combined_feature_views: set[CompiledCombinedFeatureView] = field(default_factory=set)
     models: set[Model] = field(default_factory=set)
-    enrichers: list[EnricherReference] = field(default_factory=list)
 
-    def to_dict(self, **kwargs: dict) -> dict:
+    def to_dict(self, **kwargs: dict) -> dict:  # type: ignore
         for view in self.feature_views:
             assert isinstance(view, CompiledFeatureView)
 
-        for view in self.combined_feature_views:
-            assert isinstance(view, CompiledCombinedFeatureView)
-
         for model in self.models:
             assert isinstance(model, Model)
 
-        for enricher in self.enrichers:
-            assert isinstance(enricher, EnricherReference)
         return super().to_dict(**kwargs)
 
     @staticmethod
@@ -181,7 +147,7 @@ class RepoDefinition(Codable):
     async def from_reference_at_path(repo_path: str, file_path: str) -> RepoDefinition:
         from aligned.compiler.repo_reader import RepoReader
 
-        dir_path = Path.cwd() if repo_path == '.' else Path(repo_path).absolute()
+        dir_path = Path.cwd() if repo_path == "." else Path(repo_path).absolute()
         absolute_file_path = Path(file_path).absolute()
 
         try:
@@ -190,19 +156,28 @@ class RepoDefinition(Codable):
                 logger.info(f"Loading repo from configuration '{reference.selected}'")
                 return await RepoDefinition.from_file(file)
             else:
-                logger.info('Found no configuration')
+                logger.info("Found no configuration")
         except ValueError as error:
-            logger.error(f'Error when loadin repo: {error}')
+            logger.error(f"Error when loading repo: {error}")
 
-        logger.info('Generating repo definition')
+        logger.info("Generating repo definition")
         return await RepoReader.definition_from_path(dir_path)
 
     @staticmethod
-    async def from_path(path: str) -> RepoDefinition:
+    async def from_path(
+        path: str, exclude_glob: list[str] | None = None
+    ) -> RepoDefinition:
         from aligned.compiler.repo_reader import RepoReader
 
-        dir_path = Path.cwd() if path == '.' else Path(path).absolute()
-        return await RepoReader.definition_from_path(dir_path)
+        dir_path = Path.cwd() if path == "." else Path(path).absolute()
+        return await RepoReader.definition_from_path(dir_path, exclude_glob)
+
+    @staticmethod
+    async def from_glob(glob: str, root_dir: Path | None = None) -> RepoDefinition:
+        from aligned.compiler.repo_reader import RepoReader
+
+        dir_path = Path.cwd() if root_dir is None else root_dir
+        return await RepoReader.definition_from_glob(dir_path, glob=glob)
 
     # def add_old_version(self, old_version: "RepoDefinition") -> "RepoDefinition":
 
