@@ -6,8 +6,8 @@ from dataclasses import dataclass, field
 from typing import Any
 
 import numpy as np
-import pandas as pd
 import polars as pl
+from aligned.lazy_imports import pandas as pd
 from mashumaro.types import SerializableType
 from pydantic import BaseModel, ValidationError
 
@@ -16,11 +16,11 @@ from aligned.schemas.codable import Codable
 logger = logging.getLogger(__name__)
 
 
-class SupportedTextModels:
+class SupportedEmbeddingModels:
 
-    types: dict[str, type[TextVectoriserModel]]
+    types: dict[str, type[EmbeddingModel]]
 
-    _shared: SupportedTextModels | None = None
+    _shared: SupportedEmbeddingModels | None = None
 
     def __init__(self) -> None:
         self.types = {}
@@ -28,18 +28,18 @@ class SupportedTextModels:
         for tran_type in [GensimModel, OpenAiEmbeddingModel, HuggingFaceTransformer]:
             self.add(tran_type)
 
-    def add(self, transformation: type[TextVectoriserModel]) -> None:
+    def add(self, transformation: type[EmbeddingModel]) -> None:
         self.types[transformation.name] = transformation
 
     @classmethod
-    def shared(cls) -> SupportedTextModels:
+    def shared(cls) -> SupportedEmbeddingModels:
         if cls._shared:
             return cls._shared
-        cls._shared = SupportedTextModels()
+        cls._shared = SupportedEmbeddingModels()
         return cls._shared
 
 
-class TextVectoriserModel(Codable, SerializableType):
+class EmbeddingModel(Codable, SerializableType):
     name: str
 
     @property
@@ -50,24 +50,24 @@ class TextVectoriserModel(Codable, SerializableType):
         return self.to_dict()
 
     @classmethod
-    def _deserialize(cls, value: dict) -> TextVectoriserModel:
+    def _deserialize(cls, value: dict) -> EmbeddingModel:
         name_type = value['name']
         del value['name']
-        data_class = SupportedTextModels.shared().types[name_type]
+        data_class = SupportedEmbeddingModels.shared().types[name_type]
         with suppress(AttributeError):
             if data_class.dtype:
                 del value['dtype']
 
         return data_class.from_dict(value)
 
-    async def load_model(self):
+    async def load_model(self) -> None:
         pass
 
     async def vectorise_pandas(self, texts: pd.Series) -> pd.Series:
-        pass
+        raise NotImplementedError(type(self))
 
-    async def vectorise_polars(self, texts: pl.LazyFrame, text_key: str, output_key: str) -> pl.LazyFrame:
-        pass
+    async def vectorise_polars(self, texts: pl.LazyFrame, text_key: str, output_key: str) -> pl.Expr:
+        raise NotImplementedError(type(self))
 
     @staticmethod
     def gensim(model_name: str, config: GensimConfig | None = None) -> GensimModel:
@@ -93,7 +93,7 @@ class GensimConfig(Codable):
 
 
 @dataclass
-class GensimModel(TextVectoriserModel):
+class GensimModel(EmbeddingModel):
 
     model_name: str
     config: GensimConfig = field(default_factory=GensimConfig)
@@ -170,7 +170,7 @@ class GensimModel(TextVectoriserModel):
             [pl.col(f'{text_key}_tokens').apply(vector, return_dtype=pl.List(pl.Float64)).alias(output_key)]
         )
 
-    async def load_model(self):
+    async def load_model(self) -> None:
         import gensim.downloader as gensim_downloader
 
         self.loaded_model = gensim_downloader.load(self.model_name)
@@ -187,7 +187,7 @@ class OpenAiResponse(BaseModel):
 
 
 @dataclass
-class OpenAiEmbeddingModel(TextVectoriserModel):
+class OpenAiEmbeddingModel(EmbeddingModel):
 
     api_token_env_key: str = field(default='OPENAI_API_KEY')
     model: str = field(default='text-embedding-ada-002')
@@ -238,41 +238,50 @@ class OpenAiEmbeddingModel(TextVectoriserModel):
             logger.error(f'Error parsing OpenAi Embeddings API response: {e}')
             raise e
 
-    async def load_model(self):
+    async def load_model(self) -> None:
         pass
 
     async def vectorise_pandas(self, texts: pd.Series) -> pd.Series:
         data = await self.embeddings(texts.tolist())
         return pd.Series([embedding.embedding for embedding in data.data])
 
-    async def vectorise_polars(self, texts: pl.LazyFrame, text_key: str, output_key: str) -> pl.LazyFrame:
-        data = await self.embeddings(texts.select(text_key).collect().to_series().to_list())
-        return texts.with_column(
-            pl.Series(values=[embedding.embedding for embedding in data.data], name=output_key)
-        )
+    async def vectorise_polars(self, texts: pl.LazyFrame, text_key: str, output_key: str) -> pl.Expr:
+        async def embed(text: pl.Series) -> pl.Series:
+            data = await self.embeddings(text.to_list())
+            return pl.Series([embedding.embedding for embedding in data.data])
+
+        return pl.col(text_key).map_batches(lambda text: embed(text)).alias(output_key)
 
 
 @dataclass
-class HuggingFaceTransformer(TextVectoriserModel):
+class HuggingFaceTransformer(EmbeddingModel):
 
     model: str
     name: str = 'huggingface'
     loaded_model: Any = field(default=None)
 
-    async def load_model(self):
+    @property
+    def embedding_size(self) -> int | None:
+        from sentence_transformers import SentenceTransformer
+
+        model = SentenceTransformer(self.model)
+
+        return len(model.encode(['test'])[0])
+
+    async def load_model(self) -> None:
         from sentence_transformers import SentenceTransformer
 
         self.loaded_model = SentenceTransformer(self.model)
 
-    async def vectorise_polars(self, texts: pl.LazyFrame, text_key: str, output_key: str) -> pl.LazyFrame:
+    async def vectorise_polars(self, texts: pl.LazyFrame, text_key: str, output_key: str) -> pl.Expr:
         if self.loaded_model is None:
             await self.load_model()
-        return texts.with_column(
-            pl.Series(
-                self.loaded_model.encode(texts.select(pl.col(text_key)).collect().to_series().to_list())
-            ).alias(output_key)
+
+        return (
+            pl.col(text_key)
+            .map_batches(lambda text: pl.Series(self.loaded_model.encode(text.to_list())))
+            .alias(output_key)
         )
-        pass
 
     async def vectorise_pandas(self, texts: pd.Series) -> pd.Series:
         if self.loaded_model is None:
