@@ -13,7 +13,7 @@ from pydantic import BaseModel
 
 from aligned.data_source.stream_data_source import HttpStreamSource
 from aligned.feature_source import WritableFeatureSource
-from aligned.feature_store import FeatureStore
+from aligned.feature_store import ContractStore
 from aligned.schemas.feature import Feature
 from aligned.schemas.feature_view import CompiledFeatureView
 from aligned.sources.local import StorageFileReference
@@ -35,7 +35,7 @@ class TopicInfo:
 
 class FastAPIServer:
     @staticmethod
-    def write_to_topic_path(topic: TopicInfo, feature_store: FeatureStore, app: FastAPI) -> None:
+    def write_to_topic_path(topic: TopicInfo, feature_store: ContractStore, app: FastAPI) -> None:
 
         required_features: set[Feature] = set()
         for view in topic.views:
@@ -95,7 +95,7 @@ class FastAPIServer:
             )
 
     @staticmethod
-    def feature_view_path(name: str, feature_store: FeatureStore, app: FastAPI) -> None:
+    def feature_view_path(name: str, feature_store: ContractStore, app: FastAPI) -> None:
         @app.post(f'/feature-views/{name}/all')
         async def all(limit: int | None = None) -> dict:
             df = await feature_store.feature_view(name).all(limit=limit).to_pandas()
@@ -103,11 +103,13 @@ class FastAPIServer:
             return df.to_dict('list')
 
     @staticmethod
-    def model_path(name: str, feature_store: FeatureStore, app: FastAPI) -> None:
+    def model_path(name: str, feature_store: ContractStore, app: FastAPI) -> None:
         from aligned.feature_store import RawStringFeatureRequest
 
         model = feature_store.models[name]
-        features = {f'{feature.location.identifier}:{feature.name}' for feature in model.features}
+        features = {
+            f'{feature.location.identifier}:{feature.name}' for feature in model.features.default_features
+        }
         feature_request = feature_store.requests_for(RawStringFeatureRequest(features))
 
         entities: set[Feature] = set()
@@ -146,7 +148,7 @@ class FastAPIServer:
 
         # Using POST as this can have a body with the fact / entity table
         @app.post(f'/models/{name}', openapi_extra=featch_api_schema)
-        async def get_model(entity_values: dict) -> str:
+        async def get_model(entity_values: dict) -> Response:
             missing_entities = {entity.name for entity in entities if entity.name not in entity_values}
             if missing_entities:
                 raise HTTPException(status_code=400, detail=f'Missing entity values {missing_entities}')
@@ -159,7 +161,7 @@ class FastAPIServer:
                     for value in entity_values['event_timestamp']
                 ]
 
-            df = await feature_store.model(name).features_for(entity_values).to_polars()
+            df = await feature_store.model(name).features_for(entity_values).to_lazy_polars()
             pandas_df = df.collect().to_pandas()
             orient = 'values'
             body = ','.join(
@@ -168,22 +170,15 @@ class FastAPIServer:
             return Response(content=f'{{{body}}}', media_type='application/json')
 
     @staticmethod
-    def app(feature_store: FeatureStore, auth_tokens: list[str] | None = None) -> FastAPI:
+    def app(feature_store: ContractStore, auth_tokens: list[str] | None = None) -> FastAPI:
         from asgi_correlation_id import CorrelationIdMiddleware
         from fastapi import FastAPI
         from fastapi.middleware import Middleware
-        from prometheus_fastapi_instrumentator import Instrumentator
         from starlette.status import HTTP_200_OK
 
         app = FastAPI(middleware=[Middleware(CorrelationIdMiddleware)])
         app.docs_url = '/docs'
 
-        prom_instrument = Instrumentator().instrument(app)
-
-        @app.on_event('startup')
-        async def startup():
-            prom_instrument.expose(app)
-
         @app.get('health', status_code=HTTP_200_OK)
         def health() -> None:
             return
@@ -230,10 +225,12 @@ class FastAPIServer:
             return RedirectResponse('/docs')
 
         @app.post('/features')
-        async def features(payload: APIFeatureRequest) -> dict:
+        async def features(payload: APIFeatureRequest) -> Response:
             import json
 
-            df = await feature_store.features_for(payload.entities, features=payload.features).to_polars()
+            df = await feature_store.features_for(
+                payload.entities, features=payload.features
+            ).to_lazy_polars()
             json_data = json.dumps(df.collect().to_dict(as_series=False))
             return Response(content=json_data, media_type='application/json')
 
@@ -241,7 +238,7 @@ class FastAPIServer:
 
     @staticmethod
     def run(
-        feature_store: FeatureStore,
+        feature_store: ContractStore,
         host: str | None = None,
         port: int | None = None,
         workers: int | None = None,
@@ -250,9 +247,10 @@ class FastAPIServer:
 
         app = FastAPIServer.app(feature_store)
 
-        uvicorn.run(app, host=host or '127.0.0.1', port=port or 8000, workers=workers or workers)
+        uvicorn.run(app, host=host or '127.0.0.1', port=port or 8000, workers=workers or workers)  # type: ignore
 
 
+@dataclass
 class FeatureServer:
 
     definition_reference: StorageFileReference
