@@ -2,5 +2,5 @@ class Storage:
     async def read(self, path: str) -> bytes:
         raise NotImplementedError()
 
-    async def write(self, path: str, content: bytes) -> None:
+    async def write(self, path: str, content: bytes | bytearray) -> None:
         raise NotImplementedError()
